﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using Proteus;
namespace ServiceDiagnostics
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        string _strLogPath = "";
        LogWatcher _objLogWatcher;

        public MainWindow()
        {
            InitializeComponent();

            _lblErrorText.Visibility = System.Windows.Visibility.Hidden;

            GetLogPath();
            WatchLog();
        }
        private void WatchLog()
        {
            //Begin recording log
            if (!string.IsNullOrEmpty(_strLogPath))
            {
                _objLogWatcher = new LogWatcher(_strLogPath);
                _objLogWatcher.UpdateHandler += LogUpdateHandler;
                _objLogWatcher.BeginPolling();

            }
        }
        private void GetLogPath()
        {
            string val = "";
            RegistryKey softwareKey = Registry.LocalMachine.OpenSubKey(@"Software", false);

            RegistryKey publisherKey;
            publisherKey = softwareKey.OpenSubKey("Fulmination", true);
            if (publisherKey == null)
                ShowError("Publisher key not found. App is not installed, or make sure Diag app is 64 bit.");
            else
            {
                RegistryKey appKey;
                appKey = publisherKey.OpenSubKey("Romulus Agent", false);
                if (appKey == null)
                    ShowError("App key not found. App is not installed, or make sure Diag app is 64 bit.");
                else
                {
                    val = (string)appKey.GetValue("InstallDir", "");
                    _lblAgentPath.Content = "Agent Path: " + val;
                    _strLogPath = System.IO.Path.Combine(val, "logs\\agent.log");
                    appKey.Close();
                }
                publisherKey.Close();
            }
            softwareKey.Close();
        }
        private void LogUpdateHandler(string data)
        {
            _txtLogText.Text += data;
        }
        private void ShowError(string error)
        {
            _lblErrorText.Visibility = Visibility.Visible;
            _lblErrorText.Content = "Error: " + error;
        }
        private void _btnExit_Click(object sender, RoutedEventArgs e)
        {
            Environment.Exit(0);
        }
        private void MenuItem_Click(object sender, RoutedEventArgs e)
        {
        }
        private void MenuItem_Click_1(object sender, RoutedEventArgs e)
        {
            Environment.Exit(0);
        }
        private void _mainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            _objLogWatcher.EndPolling();
        }
    }
}
