﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Proteus;
using Spartan;
using System.Reflection;
namespace BuildGui
{
    public class BuildGuiSettings
    {
        private class SettingNames
        {
            //Must be same name as variable
            public const string ExePath = "ExePath";
            public const string ProjectsFilePath = "ProjectFileName";
            public const string SolutionFilePath = "SolutionFileName";
            public const string BuildId = "BuildId";
            public const string Agents = "Agents";
            public const string ShowConsole = "ShowConsole";
            public const string MaxErrorLimit = "MaxErrorLimit";
            public const string SelectedConfigName = "SelectedConfigName";
            public const string SelectedConfigPlatform = "SelectedConfigPlatform";
            public const string JumpToError = "JumpToError";
            public const string ShowCommandWindow = "ShowCommandWindow";
            public const string ProjectName = "ProjectName";
            public const string SelectedProjectOption = "SelectedProjectOption";// Solution or rbproj project file
        }
        private class SettingDefaults
        {
            public const string ExePath = @"C:\Spartan.exe";
            public const string BuildPath = @"C:\";
            public const string BuildId = "0";
            public const string Agents = "";
            public const string ShowConsole = "false";
            public const string MaxErrorLimit = "1";
            public const string SelectedConfigName = "Debug";
            public const string SelectedConfigPlatform = "Win32";
            public const string JumpToError = "false";
            public const string ShowCommandWindow = "false";
            public const string DefaultProjectName = "MyProject";
        }
        public const string TempFilePath = @"\temp_settings.rbcfg";
        public const string ConfigFileExtension = ".rbcfg";

        public class ProjectOption {
            public const string VisualStudioSolution = "VisualStudioSolution";
            public const string RomulusProject = "RomulusProject";
        }
        #region Public:Members
        //These are the actual settings.
        public string SelectedConfigName;
        public string SelectedConfigPlatform;
        public int MaxErrorLimit;
        public string ExePath;
        public string ProjectsFilePath;
        public string SolutionFilePath;
        public string SettingsFileName;
        public string SelectedProjectOption;
        public int BuildId;
        public bool ShowConsole = true;
        public bool JumpToError = false; //If true we jump to error when build fails
        public bool ShowCommandWindow = true;
        public string ProjectName ;  // name of the project.  Removed this earlier.. why?
        #endregion

        #region Private:Members
        private List<AgentInfo> _lstAgents = new List<AgentInfo>();
        private Object _objAgentsLockObject = new Object();
        private Object _objUpdateAgentsLockObject = new Object();
        private string _strFilePath;
        private bool _blnUpdating = false;
        private Object _objIsUpdatingLockObject = new Object();
        private UserSessionPreferences _objSessionPreferences;
        private BuildGuiController _objBuildGuiController;
        #endregion

        #region Public: Props
        public UserSessionPreferences UserSessionPreferences {
            get
            {
                return _objSessionPreferences;
            }
            set
            {
                _objSessionPreferences = value;
            }
        }
        public List<AgentInfo> Agents
        {
            get
            {
                lock (_objAgentsLockObject)
                    return _lstAgents;
            }
            set
            {
                lock (_objAgentsLockObject)
                    _lstAgents = value;
            }
        }
        private BuildGuiController getManager() { return _objBuildGuiController;  }
        #endregion

        #region Public:Methods
        public BuildGuiSettings(
            BuildGuiController objController, 
            string settingsFileName = "", 
            string defaultExePath = "", 
            int defaultBuildId = 1, 
            int defaultMaxErrorLimit = 1
            )
        {
            _objBuildGuiController = objController;
            _strFilePath = TempFilePath;
            SettingsFileName = settingsFileName;
            ExePath = defaultExePath;
            BuildId = defaultBuildId;
            MaxErrorLimit = defaultMaxErrorLimit;
            SelectedConfigName = SettingDefaults.SelectedConfigName;
            SelectedConfigPlatform = SettingDefaults.SelectedConfigPlatform;
            ProjectName = SettingDefaults.DefaultProjectName;
            //If the user has a session preference file load the settings from that preference

            if(string.IsNullOrEmpty(settingsFileName) ||
                !System.IO.File.Exists(settingsFileName))
            {
                settingsFileName = string.Empty;
                CreateDefault();
            }

            //Make sure this comes after createdefault or else you will override preferences
            _objSessionPreferences = new UserSessionPreferences();
            _objSessionPreferences.Load();

            if (!string.IsNullOrEmpty(_objSessionPreferences.SettingsPath))
            {
                _strFilePath = _objSessionPreferences.SettingsPath;
                _strFilePath = ValidateFilePath(_strFilePath);
            }
        }
        //private void ValidateSettingsFilePath()
        //{
        //    // If we are the temp file path then prompt user for a valid settings path.
        //    if (_strFilePath != TempFilePath)
        //    {
        //        /* 1 - try to get the settings filename from the project name
        //         * 2 - if the project name cannot be located prompt user for settings name.
        //         */
        //        if (!System.IO.File.Exists(_strFilePath))
        //            throw new Exception("Settings could not be saved.  Please set a valid path to the project build directory in the settings tab.");
        //    }

        //}
        public void Save(string strFilePath = "")
        {
            int n = 0;
            string[] lines = new string[60];

            if (strFilePath != "")
                _strFilePath = strFilePath;

         //   ValidateSettingsFilePath();

            lines[n++] = GetConfigLine(SettingNames.ExePath, ExePath);
            lines[n++] = GetConfigLine(SettingNames.ProjectsFilePath, ProjectsFilePath);
            lines[n++] = GetConfigLine(SettingNames.SelectedProjectOption, SelectedProjectOption);
            
            lines[n++] = GetConfigLine(SettingNames.SolutionFilePath, SolutionFilePath);
            lines[n++] = GetConfigLine(SettingNames.BuildId, BuildId.ToString());
            lines[n++] = GetConfigLine(SettingNames.Agents, String.Join(",", Agents.Select(x => x.Name)));
            lines[n++] = GetConfigLine(SettingNames.ShowConsole, ShowConsole.ToString());
            lines[n++] = GetConfigLine(SettingNames.MaxErrorLimit, MaxErrorLimit.ToString());
            lines[n++] = GetConfigLine(SettingNames.SelectedConfigName, SelectedConfigName);
            lines[n++] = GetConfigLine(SettingNames.SelectedConfigPlatform, SelectedConfigPlatform);
            lines[n++] = GetConfigLine(SettingNames.JumpToError, JumpToError.ToString());
            lines[n++] = GetConfigLine(SettingNames.ShowCommandWindow, ShowCommandWindow.ToString());
            lines[n++] = GetConfigLine(SettingNames.ProjectName, ProjectName.ToString());

            System.IO.File.WriteAllLines(_strFilePath, lines);

            if (_objSessionPreferences != null)
            {
                _objSessionPreferences.SettingsPath = _strFilePath;
                _objSessionPreferences.Save();
            }
        }
        public void Load(string strFilePath = TempFilePath)
        {
            List<string> objFoundSettings = new List<string>();
            
            if (!String.IsNullOrEmpty(strFilePath) && !strFilePath.Equals(TempFilePath))
                _strFilePath = strFilePath;

            _strFilePath = ValidateFilePath(_strFilePath);

            string[] lines = System.IO.File.ReadAllLines(_strFilePath);
            foreach (string line in lines)
            {
                if (line.Trim() == string.Empty)
                    continue;

                string[] vals = line.Split(new char[] { '=' });
                vals[0] = BuildUtils.Dequote(vals[0]).Trim();
                vals[1] = BuildUtils.Dequote(vals[1]).Trim();

                objFoundSettings.Add(vals[0]);

                LoadSetting(vals[0], vals[1]);
            }

            CheckMissingSettings(objFoundSettings);

        }
        public void CreateAgent(string name)
        {
            lock (_objUpdateAgentsLockObject)
            {
                if (Agents.Where(x => x.Name.ToLower() == name.ToLower()).FirstOrDefault() == null)
                {
                    Agents.Add(
                        new AgentInfo
                        {
                            Name = name
                        }
                    );
                }

            }//end lock
        }
        public bool DeleteAgent(string name)
        {
            lock (_objUpdateAgentsLockObject)
            {
                AgentInfo inf = Agents.Where(x => x.Name.Equals(name)).FirstOrDefault();
                if (inf != null)
                {
                    Agents.Remove(inf);
                    return true;
                }
                else
                {
                    Globals.Logger.LogWarn("Could not find agent to remove.");
                    return false;
                }
            }//end lock
        }
        public void UpdateAgents()
        {
            lock (_objIsUpdatingLockObject)
            {
                if (_blnUpdating == true)
                    return;
                _blnUpdating = true;
            }

            List<AgentInfo> newAgents = new List<AgentInfo>();
            List<string> names = new List<string>();

            lock (_objUpdateAgentsLockObject)
            {
                foreach (AgentInfo inf in Agents)
                    names.Add(inf.Name);

                foreach (string infName in names)
                {
                    AgentInfo inf = AgentDiagnostics.GetAgentInfo(infName);
                    newAgents.Add(inf);
                }

                Agents = newAgents;

            }//end lock

            lock (_objIsUpdatingLockObject)
            {
                _blnUpdating = false;
            }
        }
        public BuildGuiSettings GetCopy() { return (BuildGuiSettings)this.MemberwiseClone(); }
        #endregion

        #region Private:Methods
        private void CheckMissingSettings(List<string> aobjFoundSettings)
        {
            List<string> missing = new List<string>();
            List<string> settingnames = ReflectionUtils.GetAllPublicStringFields(new SettingNames());

            foreach (string str in settingnames)
            {
                if (!aobjFoundSettings.Contains(str))
                    missing.Add(str);
            }

            if (missing.Count > 0)
            {
                foreach (string st in missing)
                {
                    string err = "Loading settings, did not find setting '" + st + "', setting to default value.";
                    Globals.Logger.LogWarn(err);
                    getManager().ShowUserFriendlyErrorMessage(err);
                }
               // Save();
            }
        }
        private string GetConfigLine(string name, string data)
        {
            return name + " = " + BuildUtils.Enquote(data);
        }
        private void LoadSetting(string key, string value)
        {
            switch (key)
            {
                case SettingNames.JumpToError:
                    JumpToError = Convert.ToBoolean(value);
                    break;
                case SettingNames.SelectedConfigName:
                    SelectedConfigName = value;
                    break;
                case SettingNames.SelectedConfigPlatform:
                    SelectedConfigPlatform = value;
                    break;
                case SettingNames.ExePath:
                    ExePath = value;
                    break;
                case SettingNames.MaxErrorLimit:
                    MaxErrorLimit = Convert.ToInt32(value);
                    break;
                case SettingNames.ProjectsFilePath:
                    ProjectsFilePath = value;
                    break;
                case SettingNames.SelectedProjectOption:
                    SelectedProjectOption = value;
                    break;
                case SettingNames.SolutionFilePath:
                    SolutionFilePath = value;
                    break;
                case SettingNames.BuildId:
                    BuildId = Convert.ToInt32(value);
                    break;
                case SettingNames.ShowConsole:
                    ShowConsole = Convert.ToBoolean(value);
                    break;
                case SettingNames.ShowCommandWindow:
                    ShowCommandWindow = Convert.ToBoolean(value);
                    break;
                case SettingNames.Agents:
                    Agents.Clear();
                    string[] vaar = value.Split(',');
                    foreach (string a in vaar)
                    {
                        if (a == string.Empty)
                            continue;
                        if (Agents.Where(x => x.Name.ToLower() == a.ToLower()).FirstOrDefault() == null)
                        {
                            CreateAgent(a);
                        }
                    }
                    break;
                case SettingNames.ProjectName:
                    ProjectName = value;
                    break;
                default:
                    //throw new NotImplementedException();
                    Globals.Logger.LogError("Invalid setting name " + key + " ignoring.");
                    break;
            }
        }
        private void CreateDefault()
        {
            // Create a default file in case we either can't load or we have a new project
            _strFilePath = TempFilePath;
            System.IO.FileStream fs = System.IO.File.OpenWrite(_strFilePath);
            fs.Close();
            fs.Dispose();
            GC.Collect();
            Save();
        }
        private string ValidateFilePath(string strPath)
        {
            string strExt = System.IO.Path.GetExtension(strPath);
            if (strExt.ToLower() != ".rbcfg")
                throw new Exception("invalid extension - tried to create a buildgui config file with " + strExt + " ");
            if (!System.IO.File.Exists(strPath))
            {
                System.Windows.MessageBox.Show("File path " + strPath + " does not exist. Settings could not be loaded. Please specify the correct file path in the settings tab.", "Error", System.Windows.MessageBoxButton.OK);
                CreateDefault();
                return TempFilePath;
            }
            return strPath;
        }
        #endregion


    }
}
