﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Proteus;
using Spartan;
namespace BuildGui
{
    // Loacl user preferences from the last session
    public class UserSessionPreferences
    {
        #region Private:class
        private class SettingNames
        {
            //Must be same name as variable
            public const string SettingsPath = "SettingsPath";
        }
        private class SettingDefaults
        {
            public const string SettingsPath = @"";
        }
        #endregion

        #region Public:Properties
        public string SettingsPath { get; set; }
        #endregion

        #region Public: Methods
        public UserSessionPreferences()
        {
            Create();
            Load();
        }
        public void Save()
        {
            int n = 0;
            string[] lines = new string[60];
            lines[n++] = GetConfigLine(SettingNames.SettingsPath, SettingsPath);
            System.IO.File.WriteAllLines(GetFileName(), lines);
        }
        public void Load()
        {
            List<string> objFoundSettings = new List<string>();

            string[] lines = System.IO.File.ReadAllLines(GetFileName());
            foreach (string line in lines)
            {
                if (line.Trim() == string.Empty)
                    continue;

                string[] vals = line.Split(new char[] { '=' });
                vals[0] = BuildUtils.Dequote(vals[0]).Trim();
                vals[1] = BuildUtils.Dequote(vals[1]).Trim();

                objFoundSettings.Add(vals[0]);

                LoadSetting(vals[0], vals[1]);
            }

        }
        #endregion

        #region private:methods
        private string GetFileName()
        {
            string userName = System.Security.Principal.WindowsIdentity.GetCurrent().Name;
            userName = userName.Replace('\\', '.');
            userName = userName.Replace('/', '.');
            string fn = "preferences." + userName + ".prefs";
            string strdir = System.IO.Directory.GetCurrentDirectory();
            string path = System.IO.Path.Combine(strdir, fn);

            return path;
        }
        private void Create()
        {
            if (!System.IO.File.Exists(GetFileName()))
            {
                System.IO.FileStream fs = System.IO.File.OpenWrite(GetFileName());
                fs.Close();
                fs.Dispose();
                GC.Collect();
                Save();
            }
        }
        private string GetConfigLine(string name, string data)
        {
            return name + " = " + BuildUtils.Enquote(data);
        }
        private void LoadSetting(string key, string value)
        {
            switch (key)
            {
                case SettingNames.SettingsPath:
                    SettingsPath = Convert.ToString(value);
                    break;
            }
        }
        #endregion

    }
}
