﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Proteus
{
    public class BuildConfig
    {
        public bool BuildConfigInitialized = false;

        public bool UseDependencyCache = false;

        // Build Intervals in milliseconds
        public ulong BuildInterval = 100;
        public ulong FileStatusQueryInterval = 500;
        public ulong ProcessorQueryTimeout = 120000; // this should probably be -1 or disabled or way huge.  This is the max time a file can compile.
        public int CompilerMaxErrorLimit = 1;
        public bool BuildProcessesAreAttachedToUI = false; // if true we kill the cmds when the UI disappears.

        // Input arguments
        //public static bool UseGlobalUncPaths = false;   //  Whether we are going to use global paths like \\dp0001\...  This is required for the dist build to work.

        public BuildPlatform BuildPlatform;
        public BuildConfiguration GlobalBuildConfiguration = new BuildConfiguration();
        public string DefaultUncRoot = "C:\\"; // appended to UNC paths if no root is specified. such as \\machine\\C$
        public string BuildConfigFileName = "build.cfg";

        // Master path where build files are stored.  This is passed in as a parameter to all programs
        //public string BuildConfigFilePath; // Location of build config file. 20151111 - removed.  We use projects file now

        // UNC capable paths
        public string CompilerBinPath;
        public string BranchRootDirectory;

        // Relative Paths
        public string GlobalBranchTempPath;//Local to branch, global to all projects
        public string GlobalBranchLibPath;//Local to branch, global to all projects
        public string GlobalBranchBinPath;//Local to branch, global to all projects

        public string BatchFileName;//filename only
        public string DepFileName; //filename only
        public string BuildLogFileName;//filename only
        public string ProjectsFileName;
        public string MachineName = System.Environment.MachineName;
        public string CoordinatorName;

        public string TryAppendUncBranchRoot(string pathIn)
        {
            if (pathIn[0] == '/' || pathIn[0] == '\\')
                pathIn = pathIn.Substring(1);
            return System.IO.Path.Combine(BranchRootDirectory, pathIn);
        }
        public string GlobalCompileOutputDirectory
        {
            get
            {
                return GlobalRootedTempPath;
            }
        }
        public string GlobalRootedBinPath
        {
            get
            {
                return
                System.IO.Path.Combine(
                    System.IO.Path.Combine(
                        BranchRootDirectory,
                        GlobalBranchBinPath
                    ),
                    GlobalBuildConfiguration.GetPathName()
                );
            }
            private set { }
        }
        public string GlobalRootedLibPath
        {
            get
            {
                return
                System.IO.Path.Combine(
                    System.IO.Path.Combine(
                        BranchRootDirectory,
                        GlobalBranchLibPath
                    ),
                    GlobalBuildConfiguration.GetPathName()
                );
            }
            private set { }
        }
        public string GlobalRootedTempPath
        {
            get
            {
                string pn = GlobalBuildConfiguration.GetPathName();
                return
                System.IO.Path.Combine(
                    System.IO.Path.Combine(
                        BranchRootDirectory,
                        GlobalBranchTempPath
                    ),
                    pn
                );
            }
            private set { }
        }

        public string GetCacheDirectory()
        {
            if (UseDependencyCache == true)
                return GlobalRootedTempPath;
            else
                return string.Empty;
        }

        private bool IsAdministrator()
        {
            System.Security.Principal.WindowsIdentity identity = System.Security.Principal.WindowsIdentity.GetCurrent();
            System.Security.Principal.WindowsPrincipal principal = new System.Security.Principal.WindowsPrincipal(identity);
            return principal.IsInRole(System.Security.Principal.WindowsBuiltInRole.Administrator);
        }
        //public static void LoadConfig()
        //{
        //    if (BuildConfigInitialized==true)
        //    {
        //        Globals.Logger.LogInfo("BuildConfig was already initialized..returning.");
        //        return;
        //    }

        //    if (string.IsNullOrEmpty(BuildConfigFilePath))
        //        Globals.Logger.LogError("Failed to load config.  the " + BuildFlags.BuildFile + " switch is required.  This tells the build where to find your project config file. Ex. /o\".\\dir\\build.cfg\".  To use a coordinator configuration specify a UNC path with \\CoordinatorMachineName\\c$\\.. ", true);

        //    string cfgPath = System.IO.Path.Combine(BuildConfigFilePath, BuildConfigFileName);

        //    if (!System.IO.File.Exists(cfgPath))
        //    {
        //        Globals.Logger.LogWarn("First file exist failed");
        //        System.IO.FileInfo fi = new System.IO.FileInfo(cfgPath);
        //        fi.Refresh();
        //        if (!fi.Exists)
        //        {
        //            Globals.Logger.LogError("IsAdmin = " 
        //                + IsAdministrator().ToString() 
        //                + "Could not find build config file '"
        //                + BuildConfigFileName
        //                + "' in directory '"
        //                + BuildConfigFilePath
        //                + "' ( " + cfgPath + ").  Engine will still try to continue to load file\n\n ", false);
        //        }
        //    }

        //    string[] lines = System.IO.File.ReadAllLines(cfgPath);
        //    foreach (string line in lines)
        //    {
        //        if (string.IsNullOrEmpty(line))
        //            continue;
        //        ParseConfigLine(line);
        //    }
        //  //  if (UseGlobalUncPaths == true)
        //   // {
        //        CompilerBinPath = MakeUncRootToCoordMachine(CompilerBinPath);
        //        BranchRootDirectory = MakeUncRootToCoordMachine(BranchRootDirectory);

        //        if (!System.IO.Directory.Exists(BranchRootDirectory))
        //            throw new Exception("Branch root directory '" + BranchRootDirectory + "' does not exist.  Please update branch root in 'build.cfg' in the build files folder. Note: The root must NOT be a UNC path if UNC is enabled in the build.cfg. ");
        //  //  }
        //  //  else
        // //       Globals.Logger.LogWarn("*Warning:UNC paths were not specified. Distributed build will fail. Set UseUncRootPath=\"true\" in build.cfg to use UNC paths.");

        //    BuildConfigInitialized = true;
        //}
        //public void ParseConfigLine(string inline, int intLineNumber)
        //{
        //    string line = BuildUtils.ParseEatComment(inline);
        //    if (string.IsNullOrEmpty(line))
        //        return;

        //    string[] values = BuildUtils.SplitValues(line, '='); ;
        //    if (values.Length != 2)
        //    {
        //        Globals.Logger.LogWarn("Invalid value '" + line + "' encountered in config file. Ignoring.");
        //        return;
        //    }

        //    string key = values[0].Trim();
        //    string value = values[1].Trim();
        //    value = BuildUtils.Dequote(value);
        //    ParseConfigLine(key, value, intLineNumber);
        //}
        private void CheckAndOverwrite(ref string val, string keyname, string newval)
        {
            if (!string.IsNullOrEmpty(val))
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        private void CheckAndOverwrite(ref int val, string keyname, int newval)
        {
            if (val!=0)
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        private void CheckAndOverwrite(ref ulong val, string keyname, ulong newval)
        {
            if (val != 0)
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        private void CheckAndOverwrite(ref bool val, string keyname, bool newval)
        {
            if (val != newval)
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        public void ParseConfigLine(string key, string value, int intLineNumber)
        {

            //b root / global config / src path
            switch (key.ToLower())
            {
                case "vcbinpath":
                    CheckAndOverwrite(ref CompilerBinPath, key, value);
                    break;
                case "branchrootdirectory":
                    CheckAndOverwrite(ref BranchRootDirectory, key, value);
                    break;
                case "tempdir":
                    CheckAndOverwrite(ref GlobalBranchTempPath, key, value);
                    break;
                case "libdir":
                    CheckAndOverwrite(ref GlobalBranchLibPath, key, value);
                    break;
                case "bindir":
                    CheckAndOverwrite(ref GlobalBranchBinPath, key, value);
                    break;
                case "batchfilename":
                    CheckAndOverwrite(ref BatchFileName, key, value);
                    break;
                case "depfilename":
                    CheckAndOverwrite(ref DepFileName, key, value);
                    break;
                case "buildlogfilename":
                    CheckAndOverwrite(ref BuildLogFileName, key, value);
                    break;
                case "projectsfilename":
                    CheckAndOverwrite(ref ProjectsFileName, key, value);
                    break;
                //case "useuncrootpath": UseGlobalUncPaths = (value.ToLower().Trim() == "true") ? true : false; break;
                case "defaultuncroot":
                    CheckAndOverwrite(ref DefaultUncRoot, key, value);
                    break;
                case "buildinterval":
                    try
                    {
                        CheckAndOverwrite(ref BuildInterval, key, System.Convert.ToUInt64(value));
                    }
                    catch (Exception ex)
                    {
                        BuildInterval = 0;
                        Globals.Logger.LogError("Build interval was less than 0, truncating to 0");
                    }
                    break;
                case "filestatusqueryinterval":
                    try
                    {
                        CheckAndOverwrite(ref FileStatusQueryInterval, key, System.Convert.ToUInt64(value));
                    }
                    catch (Exception ex)
                    {
                        FileStatusQueryInterval = 0;
                        Globals.Logger.LogError("FileStatusQueryInterval was less than 0, truncating to 0");
                    }
                    break;
                case "processorquerytimeout":
                    try
                    {
                        CheckAndOverwrite(ref ProcessorQueryTimeout, key, System.Convert.ToUInt64(value));
                    }
                    catch (Exception ex)
                    {
                        ProcessorQueryTimeout = 0;
                        Globals.Logger.LogError("ProcessorQueryTimeout was less than 0, truncating to 0");
                    }
                    break;
                case "usedependencycache":
                    CheckAndOverwrite(ref UseDependencyCache, key, (value.ToLower() == "yes") || (System.Convert.ToBoolean(value) == true));
                    break;
                default:
                    //Globals.Logger.LogWarn("Invalid value '" + line + "' encountered in config file. Ignoring.");
                    Globals.Logger.LogError("Line " + intLineNumber + ": Invalid token '" + value + "' in key '" + key + "'. Ignoring..");
                    break;
            }
        }

        public string GetMakeAndBatchFileDirectory()
        {
            return GlobalRootedTempPath;
        }
        public string GetMakeFilePath()
        {
            return System.IO.Path.Combine(GlobalRootedTempPath, BatchFileName);
        }
        public string GetBatchFilePath()
        {
            return System.IO.Path.Combine(GlobalRootedTempPath, DepFileName);
        }

        public string MakeUncRootToCoordMachine(string path)
        {
            // Adds a path root if none specified, or
            // adds the \\ unc path.

            return FileUtils.MakeUncRoot(CoordinatorName, path, DefaultUncRoot);

            //if (path[0] == '\\' && path[1] == '\\')
            //    return path;    // assume path is valid UNC root.

            //if (path[0] == '\\' || path[0] == '/')
            //    path = path.Substring(1);

            //int rlen = System.IO.Path.GetPathRoot(path).Length;
            //string root = path.Substring(0, rlen);
            //if (string.IsNullOrEmpty(root))
            //    root = DefaultUncRoot;
            //path = path.Substring(rlen, path.Length - rlen);

            //if (UseGlobalUncPaths == true)
            //{
            //    //transform to unc path.
            //    root = "\\\\" + BuildConfig.CoordinatorName + "\\" + root.Replace(':', '$');
            //}


            //return System.IO.Path.Combine(root, path);
        }

        public string ParseCoordMachine(string UncPath)
        {
            Uri uri = new Uri(UncPath);
            return uri.Host;
        }
    }
}
