﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Proteus
{
    public class FormUtils
    {
        public static void ForceWindowFocus(System.Windows.Forms.Form objForm, ulong intTimeoutMilliseconds = 1000)
        {
            ulong tA, tB;
            tA = Win32Utils.GetTickCount64();
            tB = Win32Utils.GetTickCount64();

            while (((tB - tA) < intTimeoutMilliseconds) && !objForm.Focused) {
                tB = Win32Utils.GetTickCount64();

                objForm.TopMost = true;
                objForm.TopMost = false;
                objForm.Activate();
                System.Windows.Forms.Application.DoEvents();
            }

        }
        public static void ShowErrorMessage(string msg)
        {
            System.Windows.Forms.MessageBox.Show(
                null,
                msg,
                "Error",
                System.Windows.Forms.MessageBoxButtons.OK,
                System.Windows.Forms.MessageBoxIcon.Exclamation,
                System.Windows.Forms.MessageBoxDefaultButton.Button1
                );
            Globals.Logger.LogError(msg);
        }
        public static string GetValidUserOpenFile(bool multiselect = false, string initialdir = "")
        {
            System.Windows.Forms.OpenFileDialog dlg = new System.Windows.Forms.OpenFileDialog();
            dlg.Multiselect = multiselect;
            if (initialdir == "")
                dlg.InitialDirectory = System.IO.Directory.GetCurrentDirectory();

            System.Windows.Forms.DialogResult res = dlg.ShowDialog();

            if (res == System.Windows.Forms.DialogResult.Cancel)
                return string.Empty;

            if (!System.IO.File.Exists(dlg.FileName))
            {
                ShowErrorMessage("Selected file " + dlg.FileName 
                    + " could not be found, or Romulus no permissions to access this path.  Romulus may need to be run as Administrator.");
                return string.Empty;
            }

            return dlg.FileName;
        }
        public static string GetValidUserSaveFile()
        {
            System.Windows.Forms.SaveFileDialog dlg = new System.Windows.Forms.SaveFileDialog();
            System.Windows.Forms.DialogResult res = dlg.ShowDialog();

            if (res == System.Windows.Forms.DialogResult.Cancel)
                return string.Empty;

            if (!System.IO.File.Exists(dlg.FileName))
            {
                ShowErrorMessage("Selected file " + dlg.FileName
                    + " could not be found, or Romulus no permissions to access this path.  Romulus may need to be run as Administrator.");
                return string.Empty;
            }

            return dlg.FileName;
        }
        public static string GetValidUserPath()
        {
            System.Windows.Forms.FolderBrowserDialog dlg = new System.Windows.Forms.FolderBrowserDialog();
            System.Windows.Forms.DialogResult res = dlg.ShowDialog();

            if (res == System.Windows.Forms.DialogResult.Cancel)
                return string.Empty;

            if (!System.IO.Directory.Exists(dlg.SelectedPath))
            {
                ShowErrorMessage("Selected path " + dlg.SelectedPath 
                    + " could not be found, or Romulus no permissions to access this path.  Romulus may need to be run as Administrator.");
                return string.Empty;
            }

            return dlg.SelectedPath;
        }
    }
}
