﻿using System;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Text.RegularExpressions;
namespace Proteus
{
    public enum NetworkMode { Binary, Text }

    public class NetworkUtils
    {
        #region PARSING

        static Encoding _objBinaryEncoding = Encoding.GetEncoding("iso-8859-1");

        public const string IntegerSizeMask = "000000000000";
        public const string LongSizeMask = "000000000000000000000000";
        public const int SizeofLong = 8;


        public static byte[] GetBytes(string str)
        {
            return _objBinaryEncoding.GetBytes(str);
        }
        public static string GetString(byte[] b, int start=0, int len=-1)
        {
            if (len == -1)
                len = b.Length;
            return _objBinaryEncoding.GetString(b, 0, len);
        }
        public static string PackCrc32(string packet, NetworkMode eNetworkMode)
        {
            uint crc = Crc32.Compute(packet);
            return PackUint(crc, eNetworkMode);
        }
        public static bool UnpackAndCheckCrc32(ref string packetWithCrc32Appended, NetworkMode eNetworkMode)
        {
            // Returns false if the CRC32 failed, or the packet was malformed.

            if(eNetworkMode==NetworkMode.Binary)
            {
                if (packetWithCrc32Appended.Length - 4 < 0)
                    return false;
                string CrcUInt = packetWithCrc32Appended.Substring(packetWithCrc32Appended.Length - 4, 4);
                string packetWithoutCrc = packetWithCrc32Appended.Substring(0, packetWithCrc32Appended.Length - 4);

                uint crcUnpacked = UnpackUint(ref CrcUInt, eNetworkMode);
                uint crcComputed = Crc32.Compute(packetWithoutCrc);

                if (crcUnpacked != crcComputed)
                    return false;

                return true;
            }
            else if(eNetworkMode==NetworkMode.Text)
            {
                if (packetWithCrc32Appended.Length - IntegerSizeMask.Length < 0)
                    return false;

                string CrcUInt = packetWithCrc32Appended.Substring(packetWithCrc32Appended.Length - IntegerSizeMask.Length, IntegerSizeMask.Length);
                string packetWithoutCrc = packetWithCrc32Appended.Substring(0, packetWithCrc32Appended.Length - IntegerSizeMask.Length);
                uint crcUnpacked = System.Convert.ToUInt32(CrcUInt);
                uint crcComputed = Crc32.Compute(packetWithoutCrc);

                if (crcUnpacked != crcComputed)
                    return false;

                return true;
            }
            throw new NotImplementedException();
        }
        public static NetworkPacketType UnpackPacketType(ref string data, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Text)
            {
                string strCode = ParseDataChunk(ref data, NetworkSettings.HeaderLength);
                return NetworkSettings.HeaderStringToPacketType(strCode);
            }
            else
                throw new NotImplementedException();
        }
        public static string PackPacketType(NetworkPacketType npt, NetworkMode eNetworkMode)
        {
            return NetworkSettings.PacketTypeToHeaderString(npt, eNetworkMode);
        }
        public static string PackString(string str, NetworkMode eNetworkMode)
        {
                if (str == null)
                    str = "";
                return GetStringHeader(str, eNetworkMode) + str;
        }
        public static string PackBytes(byte[] data, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Text)
            {
                string str = _objBinaryEncoding.GetString(data);
                return PackString(str, eNetworkMode);
            }
            else
                throw new NotImplementedException();
        }
        public static string GetStringHeader(string str, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Text)
            {
                return str.Length.ToString(IntegerSizeMask);
            }
            else if(eNetworkMode == NetworkMode.Binary)
            {
                return PackLong((long)str.Length, eNetworkMode);
            }
            else throw new NotImplementedException();
        }
        public static string UnpackString(ref string buffer, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Text)
            {
                return ReadStringFromStream(ref buffer, eNetworkMode);
            }
            else if (eNetworkMode == NetworkMode.Binary)
            {
                ulong val = NetworkUtils.UnpackULong(ref buffer, eNetworkMode);
                return NetworkUtils.ParseDataChunk(ref buffer, (int)val);
            }
            else
                throw new NotImplementedException();
        }
        public static string ReadStringFromStream(ref string buffer, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Text)
            {
                //Read Len
                string strLen = ReadFixedStringFromStream(ref buffer, IntegerSizeMask.Length, eNetworkMode);
                int iStringLen;
                if (strLen == "")
                {
                    int n = 0;
                    n++;
                }
                iStringLen = Convert.ToInt32(strLen);

                // Read Data
                return ReadFixedStringFromStream(ref buffer, iStringLen, eNetworkMode);
            }
            else
                throw new NotImplementedException();
        }
        public static string ReadFixedStringFromStream(ref string buffer, int len, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Text)
            {
                return NetworkUtils.ParseDataChunk(ref buffer, len);
            }
            else
                throw new NotImplementedException();
        }
        public static string ParseDataChunk(ref string strBuffer, int iChunkSize)
        {
            string ret;

            if (strBuffer.Length < iChunkSize)
                iChunkSize = strBuffer.Length;

            ret = strBuffer.Substring(0, iChunkSize);
            strBuffer = strBuffer.Substring(iChunkSize, strBuffer.Length - iChunkSize);

            return ret;
        }
        public static string PackUint(uint i, NetworkMode eNetworkMode)
        {
            if(eNetworkMode == NetworkMode.Binary)
            {
                return _objBinaryEncoding.GetString(BitConverter.GetBytes(i));
            }
            else if(eNetworkMode == NetworkMode.Text)
            {
                return i.ToString(IntegerSizeMask);
            }
            throw new NotImplementedException();
        }
        public static string PackInt(int i, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Binary)
            {
                return _objBinaryEncoding.GetString(BitConverter.GetBytes(i));
            }
            else if (eNetworkMode == NetworkMode.Text)
            {
                return i.ToString(IntegerSizeMask);
            }
            throw new NotImplementedException();
        }
        public static string PackLong(long i, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Binary)
            {
                return _objBinaryEncoding.GetString(BitConverter.GetBytes(i));
            }
            else if (eNetworkMode == NetworkMode.Text)
            {
                return i.ToString(LongSizeMask);
            }
            throw new NotImplementedException();
        }
        public static int UnpackInt(ref string str, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Binary)
            {
                string istr = ParseDataChunk(ref str, 4);
                int val2 = BitConverter.ToInt32(_objBinaryEncoding.GetBytes(istr), 0);
                return val2;
            }
            else if (eNetworkMode == NetworkMode.Text)
            {
                string istr = ParseDataChunk(ref str, IntegerSizeMask.Length);
                return System.Convert.ToInt32(istr);
            }
            throw new NotImplementedException(); ;
        }
        public static uint UnpackUint(ref string str, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Binary)
            {
                string istr = ParseDataChunk(ref str, 4);
                uint val2 = BitConverter.ToUInt32(_objBinaryEncoding.GetBytes(istr), 0);
                return val2;
            }
            else if (eNetworkMode == NetworkMode.Text)
            {
                string istr = ParseDataChunk(ref str, IntegerSizeMask.Length);
                return System.Convert.ToUInt32(istr);
            }
            throw new NotImplementedException(); ;
        }
        public static ulong UnpackULong(ref string str, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Binary)
            {
                string istr = ParseDataChunk(ref str, 8);
                ulong val2 = BitConverter.ToUInt64(_objBinaryEncoding.GetBytes(istr), 0);
                return val2;
            }
            //not implemented in text mode
            throw new NotImplementedException(); ;
        }
        public static string PackDateTime(DateTime dt, NetworkMode eNetworkMode)
        {
            if (eNetworkMode == NetworkMode.Text)
            {
                return String.Format("{0,48:F}", dt);
            }
            else
                //not implemented yet
                throw new NotImplementedException();
        }
        public static DateTime UnpackDateTime(ref string buffer, NetworkMode eNetworkMode)
        {
            DateTime dt;
            if (eNetworkMode == NetworkMode.Text)
            {
                string part = ParseDataChunk(ref buffer, 48);
                dt = DateTime.Parse(part.Trim());
            }
            else
                //not implemented yet
                throw new NotImplementedException();
            return dt;
        }

        #endregion

        #region TCPIP
        public static System.Net.IPAddress GetIpAddress(string machineName = "")
        {
            System.Net.IPAddress ip = null;
            System.Net.IPHostEntry hostEntry;

            if (machineName == "")
                machineName = System.Environment.MachineName;

            hostEntry = System.Net.Dns.GetHostEntry(machineName);

            if (hostEntry == null)
                Globals.Logger.LogError("Failed to find host enrty for computer name:" + machineName);

            for (int n = 0; n < hostEntry.AddressList.Length; n++)
                if (hostEntry.AddressList[n].AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                    ip = hostEntry.AddressList[n];

            if (ip == null)
                Globals.Logger.LogError("Failed to find ip address for computer host:" + machineName);

            return ip;
        }
        #endregion


        #region HTML
        public static string GetHtmlDataFromUrlViaSockets(string url)
        {
            Socket socket;
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            socket.Connect(url, 80);
            string GETrequest = "GET / HTTP/1.1\r\nHost: " + GetIpAddress().ToString() + "\r\nConnection: keep-alive\r\nAccept: text/html\r\nUser-Agent: Mozilla-Firefox\r\n\r\n";
            socket.Send(Encoding.ASCII.GetBytes(GETrequest));

            bool flag = true; // just so we know we are still reading
            string headerString = ""; // to store header information
            int contentLength = 0; // the body length
            byte[] bodyBuff = new byte[0]; // to later hold the body content
            while (flag)
            {
                // read the header byte by byte, until \r\n\r\n
                byte[] buffer = new byte[1];
                socket.Receive(buffer, 0, 1, 0);
                headerString += Encoding.ASCII.GetString(buffer);
                if (headerString.Contains("\r\n\r\n"))
                {
                    // header is received, parsing content length
                    // I use regular expressions, but any other method you can think of is ok
                    Regex reg = new Regex("\\\r\nContent-Length: (.*?)\\\r\n");
                    Match m = reg.Match(headerString);
                    contentLength = int.Parse(m.Groups[1].ToString());
                    flag = false;
                    // read the body
                    bodyBuff = new byte[contentLength];
                    socket.Receive(bodyBuff, 0, contentLength, 0);
                }
            }
            return Encoding.ASCII.GetString(bodyBuff);
        }
        private class MyWebClient : WebClient
        {
            protected override WebRequest GetWebRequest(Uri uri)
            {
                WebRequest w = base.GetWebRequest(uri);
                w.Timeout = 3000;
                return w;
            }
        }
        public static string GetHtmlDataFromUrl(string url)
        {
            string ContentHtml = string.Empty ;
            if (!NetworkUtils.ValidateUrl(url))
                return "";
            try
            {
                //ContentHtml = new System.Net.WebClient().DownloadString(url);

                MyWebClient wc = new MyWebClient();
                wc.Headers.Add("user-agent", "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:39.0) Gecko/20100101 Firefox/39.0");
                
                System.IO.Stream resStream = wc.OpenRead(url);
                System.IO.StreamReader sr = new System.IO.StreamReader(resStream, System.Text.Encoding.Default);
                ContentHtml = sr.ReadToEnd();

                resStream.Close();
                //sr.Close();
            }
            catch (Exception ex)
            {
                Globals.Logger.LogError(ex.ToString(),false);
            }
            return ContentHtml;
        }

        public static bool ValidateUrl(string url)
        {
            Uri result = null;
            Uri.TryCreate(url, UriKind.RelativeOrAbsolute, out result);
            if (result != null)
            {
                try
                {
                    string scheme = result.Scheme;
                    return (scheme == Uri.UriSchemeHttp || scheme == Uri.UriSchemeHttps);
                }
                catch (Exception ex)
                {
                    return false;
                }
            }
            return false;
        }
        #endregion
    }
}
