﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Proteus
{
    public class PacketMakerBase
    {
        #region Protected: Members
        protected class RecvStateObj
        {
            public byte[] buf = new byte[2048];
            public System.Net.Sockets.Socket sock;
        }
        protected class WorkingPacket
        {
            public Object _objLockObject = new Object();
            public int _intLastPacketSize = -1;
            public string _strRecvBuffer = "";
        }

        #endregion
        #region Protected: Members

        protected static int PacketCrcSizeBytes = 4;
        // protected static string PacketTerminator = "~END\0\0\0\0";
        protected WorkingPacket _objWorkingPacket = new WorkingPacket();
        protected Queue<string> _lstPacketsText = new Queue<string>();
        protected Queue<Packet> _lstPacketsBinary = new Queue<Packet>();
        protected Object _objPacketsLockObject = new Object();
        private NetworkMode _eNetworkMode;
        private long _iBinaryOrdinal = 0; //Ordinal of when the packet was sent from this client;

        #endregion

        public NetworkMode GetNetworkMode() { return _eNetworkMode; }

        public PacketMakerBase(NetworkMode eNetworkMode)
        {
            _eNetworkMode = eNetworkMode;
        }
        public string DequeuePacketText()
        {
            string st;
            lock (_objPacketsLockObject)
            {
                if (_lstPacketsText.Count == 0)
                    return "";
                st = _lstPacketsText.Dequeue();
            }
            return st;
        }
        public Packet DequeuePacketBinary()
        {
            Packet st = null;
            lock (_objPacketsLockObject)
            {
                if (_lstPacketsBinary.Count == 0)
                    return null;
                st = _lstPacketsBinary.Dequeue();
            }
            return st;
        }
        //Changed this from "static" per each client using an ordinal value to send packets.
        public byte[] MakePacket(string strPacket, string strPrependTag, NetworkMode eNetworkMode)
        {
            string crc, packSize;
            string finalPacket;
            string strPacketWithoutCrc;

            //*Add Footer.  Packet size does not count the 12 integer bytes prepended to the packet
            //**NOTE: packet length does NOT include prepended tags.
            if (eNetworkMode == NetworkMode.Text)
            {
                crc = NetworkUtils.PackCrc32(strPacket, eNetworkMode);// Packs the CRC as a typically packed UINT
                packSize = NetworkUtils.PackInt(strPacket.Length + crc.Length, eNetworkMode);
                finalPacket = strPrependTag + packSize + strPacket + crc;
            }
            else
            {
                string ordinal = NetworkUtils.PackLong(_iBinaryOrdinal, eNetworkMode);
                if (_iBinaryOrdinal > 1000000)
                {
                    int ohFuck = 0;
                    ohFuck++;
                }
                int iPackSize_Size = 8; //size of the pack size value
                int iPackSize_Crc32 = 4;
                packSize = NetworkUtils.PackLong(strPrependTag.Length + iPackSize_Size + ordinal.Length + strPacket.Length + iPackSize_Crc32, eNetworkMode);

                _iBinaryOrdinal++;

                strPacketWithoutCrc = strPrependTag + packSize + ordinal + strPacket;

                crc = NetworkUtils.PackCrc32(strPacketWithoutCrc, eNetworkMode);// Packs the CRC as a typically packed UINT

                finalPacket = strPacketWithoutCrc + crc;
            }
            //[packsize][ [packet] [crc] ]


            if (eNetworkMode == NetworkMode.Binary)
                if (finalPacket.Length < 30)
                    throw new Exception("Packet size must be at least 30 bytes for binary packets: Tag(2) Len(8) head(8) ordinal(8), crc(4)");

            byte[] ret = NetworkUtils.GetBytes(finalPacket);

            return ret;
        }
        protected void AddReceivedDataToPacket(string buf)
        {
            if (_eNetworkMode == NetworkMode.Text)
            {
                AddReceivedDataToPacketText(buf);
            }
            else if (_eNetworkMode == NetworkMode.Binary)
            {
                AddReceivedDataToPacketBinray(buf);
            }
            else
                throw new NotImplementedException();
        }
        protected void AddReceivedDataToPacketBinray(string buf)
        {
            //[Tag (2)][size(8) ][ord (8)][head (8)][data ...][CRC (4)]
            lock (_objWorkingPacket._objLockObject)
            {
                //loop through buffer
                while (buf.Length > 0)//for (int ch = 0; ch < buf.Length; ch++)
                {
                    //check for new packet
                    //check for new packet
                    if (_objWorkingPacket._intLastPacketSize == -1)
                    {
                        //"VT" + Size
                        if(_objWorkingPacket._strRecvBuffer.Length >= 2 + 8)
                        {
                            string buf2 = _objWorkingPacket._strRecvBuffer.Substring(2, 8);
                            _objWorkingPacket._intLastPacketSize = NetworkUtils.UnpackInt(ref buf2, _eNetworkMode);
                        }
                    }

                    // ** Loop byte for byte and check the packets.
                    CheckAddPacketBinary();
                    
                    if (buf.Length > 0)
                    {
                        _objWorkingPacket._strRecvBuffer += buf[0];
                        buf = buf.Substring(1);
                    }
                    
                }
                //This here is critical - we must check after the loop because the count may equal - the loop exits and 
                // we get a stuck packet.
                CheckAddPacketBinary();
            }
        }
        private bool CheckAddPacketBinary()
        {
            Packet objPacket = new Packet();

            if (_objWorkingPacket._intLastPacketSize <= 0)
                return false ;
            if (_objWorkingPacket._strRecvBuffer.Length < _objWorkingPacket._intLastPacketSize)
                return false;

            string strPackData = _objWorkingPacket._strRecvBuffer.Substring(0,_objWorkingPacket._intLastPacketSize);
            _objWorkingPacket._strRecvBuffer = _objWorkingPacket._strRecvBuffer.Substring(_objWorkingPacket._intLastPacketSize);
            //Get here - we might have a packet

            if (!NetworkUtils.UnpackAndCheckCrc32(ref strPackData, _eNetworkMode))
            {
                Globals.Logger.LogError("Packet CRC32 Failed (2): " + strPackData);
                return false;
            }

            //[Tag (2)][size(8) ][ord (8)][head (8)][data ...][CRC (4)]
            strPackData = strPackData.Substring(2, strPackData.Length - 2);
            ulong size = NetworkUtils.UnpackULong(ref strPackData, _eNetworkMode);
            objPacket.Ordinal = NetworkUtils.UnpackULong(ref strPackData, _eNetworkMode);
            objPacket.Head = (NetworkPacketType)NetworkUtils.UnpackULong(ref strPackData, _eNetworkMode);
            objPacket.Data = strPackData.Substring(0, strPackData.Length - 4);//Data without the CRC at the end

            // **All good, enqueue packet
            lock (_objPacketsLockObject)
            {
                _lstPacketsBinary.Enqueue(objPacket);
            }
            return true;
        }
        protected void AddReceivedDataToPacketText(string buf)
        {
            lock (_objWorkingPacket._objLockObject)
            {
                //loop through buffer
                while (buf.Length > 0)//for (int ch = 0; ch < buf.Length; ch++)
                {
                    //check for new packet
                    if (_objWorkingPacket._intLastPacketSize == -1)
                    {
                        _objWorkingPacket._intLastPacketSize = NetworkUtils.UnpackInt(ref buf, _eNetworkMode);
                    }

                    // ** Loop byte for byte and check the packets.
                    if (CheckAddPacketText() == false)
                    {
                        if (buf.Length > 0)
                        {
                            _objWorkingPacket._strRecvBuffer += buf[0];
                            buf = buf.Substring(1);
                        }
                    }
                }
                //This here is critical - we must check after the loop because the count may equal - the loop exits and 
                // we get a stuck packet.
                CheckAddPacketText();
            }
        }
        private bool CheckAddPacketText()
        {
            //Check to see if the buffer has a packet.
            string strPacket;

            if (_objWorkingPacket._strRecvBuffer.Length != _objWorkingPacket._intLastPacketSize)
                return false;

            strPacket = _objWorkingPacket._strRecvBuffer;

            //Reset packet buffer
            _objWorkingPacket._strRecvBuffer = "";
            _objWorkingPacket._intLastPacketSize = -1;

            // The packet had no footer.
            if ((strPacket.Length - PacketCrcSizeBytes) < 0)
            {
                Globals.Logger.LogError("RCV INvalid - Packet had no footer:" + strPacket);
                return false;
            }

            if (!NetworkUtils.UnpackAndCheckCrc32(ref strPacket, _eNetworkMode))
            {
                Globals.Logger.LogError("Packet CRC32 Failed: " + strPacket);
                return false;
            }

            // **All good, enqueue packet
            lock (_objPacketsLockObject)
            {
                _lstPacketsText.Enqueue(strPacket);
            }
            return true;
        }
    }
}
