﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Proteus;
namespace Spartan
{
    public class BuildConfig
    {
        public bool BuildConfigInitialized = false;

        public bool UseDependencyCache = false;

        // Build Intervals in milliseconds
        public ulong CoordinatorBuildInterval = 100;
        public ulong FileStatusQueryInterval = 500;
        public ulong ProcessorQueryTimeout = 120000; // this should probably be -1 or disabled or way huge.  This is the max time a file can compile.
        public int CompilerMaxErrorLimit = 1;
        public bool BuildProcessesAreAttachedToUI = false; // if true we kill the cmds when the UI disappears.
        public BuildPlatform BuildPlatform; // Win32 or Linux64, or Win64 ... 
        public BuildConfiguration GlobalBuildConfiguration = new BuildConfiguration();
        public string DefaultUncRoot = "C:\\"; // appended to UNC paths if no root is specified. such as \\machine\\C$
        public string BuildConfigFileName = "build.cfg";

        //**These values are copied form the RomulusCommandLineArgs        
        public List<AgentInfo> CoordinatorAgentInfos = null;
        public ProgramFunction ProgramFunction = ProgramFunction.None;
        public List<CompileOption> CompileOptions = new List<CompileOption>();
        public int CurrentBuildId; // The id of the CURRENT build being executed

        public string _strBuildLogFilePath;
        public string BuildLogFilePath {
            get { return _strBuildLogFilePath; }
            set { _strBuildLogFilePath = value; }
        }

        // UNC capable paths
        public string CompilerBinPath;
        public string BranchRootDirectory; // This is NOT loaded from the build config.  Instead we use the location of the rbproj file.

        // Relative Paths
        public string GlobalBranchTempPath;//Local to branch, global to all projects
        public string GlobalBranchLibPath;//Local to branch, global to all projects
        public string GlobalBranchBinPath;//Local to branch, global to all projects

        public string BatchFileName;//filename only
        public string DepFileName; //filename only
        public string BuildLogFileName;//filename only
        public string MachineName = System.Environment.MachineName;
        public string CoordinatorName;

        public string GetBuildLogFilePath(int buildId)
        {
            string dirName = BuildLogFilePath;
            string ret = System.IO.Path.Combine(dirName, GetBuildLogFileName(buildId));
            return ret;
        }
        public string GetBuildLogFileName(int buildId)
        {
            string fileName = string.Empty;
            if (buildId < 0)
            {
                fileName = "build_" + Win32Utils.GetTickCount64() + ".log";
            }
            else
            {
                fileName = "build_" + String.Format("{0:0000000000}", buildId) + ".log";
            }
            return fileName;
        }

        public string TryAppendUncBranchRoot(string pathIn)
        {
            if (pathIn[0] == '/' || pathIn[0] == '\\')
                pathIn = pathIn.Substring(1);
            return System.IO.Path.Combine(BranchRootDirectory, pathIn);
        }
        public string GlobalCompileOutputDirectory
        {
            get
            {
                return GlobalRootedTempPath;
            }
        }
        public string GlobalRootedBinPath
        {
            get
            {
                return
                System.IO.Path.Combine(
                    System.IO.Path.Combine(
                        BranchRootDirectory,
                        GlobalBranchBinPath
                    ),
                    GlobalBuildConfiguration.GetPathName()
                );
            }
            private set { }
        }
        public string GlobalRootedLibPath
        {
            get
            {
                return
                System.IO.Path.Combine(
                    System.IO.Path.Combine(
                        BranchRootDirectory,
                        GlobalBranchLibPath
                    ),
                    GlobalBuildConfiguration.GetPathName()
                );
            }
            private set { }
        }
        public string GlobalRootedTempPath
        {
            get
            {
                string pn = GlobalBuildConfiguration.GetPathName();
                return
                System.IO.Path.Combine(
                    System.IO.Path.Combine(
                        BranchRootDirectory,
                        GlobalBranchTempPath
                    ),
                    pn
                );
            }
            private set { }
        }

        public string GetCacheDirectory()
        {
            if (UseDependencyCache == true)
                return GlobalRootedTempPath;
            else
                return string.Empty;
        }

        private bool IsAdministrator()
        {
            System.Security.Principal.WindowsIdentity identity = System.Security.Principal.WindowsIdentity.GetCurrent();
            System.Security.Principal.WindowsPrincipal principal = new System.Security.Principal.WindowsPrincipal(identity);
            return principal.IsInRole(System.Security.Principal.WindowsBuiltInRole.Administrator);
        }
        private void CheckAndOverwrite(ref string val, string keyname, string newval)
        {
            if (!string.IsNullOrEmpty(val))
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        private void CheckAndOverwrite(ref int val, string keyname, int newval)
        {
            if (val!=0)
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        private void CheckAndOverwrite(ref ulong val, string keyname, ulong newval)
        {
            if (val != 0)
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        private void CheckAndOverwrite(ref bool val, string keyname, bool newval)
        {
            if (val != newval)
                Globals.Logger.LogWarn(keyname + " was already defined as " + val + " overwriting with " + newval + ". ");
            val = newval;
        }
        public void ParseConfigLine(string key, string value, int intLineNumber)
        {

            //b root / global config / src path
            switch (key.ToLower())
            {
                case "vcbinpath":
                    CheckAndOverwrite(ref CompilerBinPath, key, value);
                    break;
                case "branchrootdirectory":
                    throw new Exception("Branch root directory is no longer a valid setting for the build config file.  The root directory is the rbproj project file path.");
                   // CheckAndOverwrite(ref BranchRootDirectory, key, value);
                  //  break;
                case "tempdir":
                    CheckAndOverwrite(ref GlobalBranchTempPath, key, value);
                    break;
                case "libdir":
                    CheckAndOverwrite(ref GlobalBranchLibPath, key, value);
                    break;
                case "bindir":
                    CheckAndOverwrite(ref GlobalBranchBinPath, key, value);
                    break;
                case "batchfilename":
                    CheckAndOverwrite(ref BatchFileName, key, value);
                    break;
                case "depfilename":
                    CheckAndOverwrite(ref DepFileName, key, value);
                    break;
                case "buildlogfilename":
                    CheckAndOverwrite(ref BuildLogFileName, key, value);
                    break;
                case "projectsfilename":
                    //CheckAndOverwrite(ref ProjectsFileName, key, value);
                    throw new Exception("ProjectsFileName is no longer a valid setting in the build config file.");
                    //break;
                //case "useuncrootpath": UseGlobalUncPaths = (value.ToLower().Trim() == "true") ? true : false; break;
                case "defaultuncroot":
                    CheckAndOverwrite(ref DefaultUncRoot, key, value);
                    break;
                case "buildinterval":
                    try
                    {
                        CheckAndOverwrite(ref CoordinatorBuildInterval, key, System.Convert.ToUInt64(value));
                    }
                    catch (Exception ex)
                    {
                        CoordinatorBuildInterval = 0;
                        Globals.Logger.LogError("Build interval was less than 0, truncating to 0");
                    }
                    break;
                case "filestatusqueryinterval":
                    try
                    {
                        CheckAndOverwrite(ref FileStatusQueryInterval, key, System.Convert.ToUInt64(value));
                    }
                    catch (Exception ex)
                    {
                        FileStatusQueryInterval = 0;
                        Globals.Logger.LogError("FileStatusQueryInterval was less than 0, truncating to 0");
                    }
                    break;
                case "processorquerytimeout":
                    try
                    {
                        CheckAndOverwrite(ref ProcessorQueryTimeout, key, System.Convert.ToUInt64(value));
                    }
                    catch (Exception ex)
                    {
                        ProcessorQueryTimeout = 0;
                        Globals.Logger.LogError("ProcessorQueryTimeout was less than 0, truncating to 0");
                    }
                    break;
                case "usedependencycache":
                    CheckAndOverwrite(ref UseDependencyCache, key, (value.ToLower() == "yes") || (System.Convert.ToBoolean(value) == true));
                    break;
                default:
                    //Globals.Logger.LogWarn("Invalid value '" + line + "' encountered in config file. Ignoring.");
                    Globals.Logger.LogError("Line " + intLineNumber + ": Invalid token '" + value + "' in key '" + key + "'. Ignoring..");
                    break;
            }
        }

        public string GetMakeAndBatchFileDirectory()
        {
            return GlobalRootedTempPath;
        }
        public string GetMakeFilePath()
        {
            return System.IO.Path.Combine(GlobalRootedTempPath, BatchFileName);
        }
        public string GetBatchFilePath()
        {
            return System.IO.Path.Combine(GlobalRootedTempPath, DepFileName);
        }

        public string MakeUncRootToCoordMachine(string path)
        {
            // Adds a path root if none specified, or
            // adds the \\ unc path.
            return FileUtils.MakeUncRoot(CoordinatorName, path, DefaultUncRoot);
        }

        public string ParseCoordMachine(string UncPath)
        {
            Uri uri = new Uri(UncPath);
            return uri.Host;
        }
    }
}
