﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Proteus;
namespace Spartan
{
    /// <summary>
    /// encapsulats command line arguments
    /// </summary>
    public class RomulusCommandLineArgs
    {
        public string ProjectFilePath;
        public string ConfigName;
        public string PlatformName;
        public int ErrorLimit;
        public bool AttachedToUi;

        //public BuildConfiguration BuildConfiguration = BuildConfiguration.Release;
        public ProgramFunction ProgramFunction = ProgramFunction.None;
        public List<CompileOption> CompileOptions = new List<CompileOption>();

        public int UserSuppliedBuildId = -1;
        public List<AgentInfo> CoordinatorAgentInfos = new List<AgentInfo>();// ** Used by coordinator only

        public List<string> Errors = new List<string>();
        public RomulusCommandLineArgs()
        {
        }
        public RomulusCommandLineArgs(List<string> args)
        {
            ParseArgs(args);
        }

        /// <summary>
        /// Prase command line arguments for th e whole romulus build systme
        /// Reutrns a list of errors
        /// </summary>
        /// <param name="args"></param>
        /// <returns></returns>
        public void ParseArgs(List<string> args)
        {
            string temp;
            Errors = new List<string>();
            foreach (string arg in args)
            {
                temp = "";

                if (StringUtils.ParseCmdArg(arg, BuildFlags.ConfigPlatform, ref temp))
                {
                    PlatformName = temp;
                }
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.ConfigName, ref temp))
                {
                    ConfigName = temp;
                }
                //else if (StringUtils.ParseCmdArg(arg, BuildFlags.BuildDir, ref temp))
                //{
                //    BuildConfigFilePath = temp;
                //    if (String.IsNullOrEmpty(BuildConfigFilePath))
                //        throw new Exception(@"Build config file path (" + BuildFlags.BuildFile + ":""path"") was not set.");
                //    CoordinatorName = ParseCoordMachine(BuildConfigFilePath);
                //    if (String.IsNullOrEmpty(CoordinatorName))
                //        throw new Exception(@"Coordinator name was not found in the /o switch while parsing the UNC path.\n" +
                //            @" Please make sure that the build path is a UNC path, for example '\\MyPCName\C$\dev\build'.  (arg = " + arg + ")");
                //}
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.ProjectFile, ref temp))
                {
                    ProjectFilePath = temp;
                    if (String.IsNullOrEmpty(ProjectFilePath))
                        throw new Exception(@"Build config file path (" + BuildFlags.ProjectFile + ":path) was not set.");
                }
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.MaxErrorLimit, ref temp))
                {
                    ErrorLimit = Convert.ToInt32(temp);
                    if (ErrorLimit <= 0)
                    {
                        Globals.Logger.LogWarn("[BuildConfig] invalid error limit found:" + temp + ", setting to default of 1.");
                        ErrorLimit = 1;
                    }
                }
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.AttachBuildProcessesToUI, ref temp))
                {
                    AttachedToUi = Convert.ToBoolean(temp);
                }
                // *****Copie dfrom spartan command line
                // *****Copie dfrom spartan command line
                // *****Copie dfrom spartan command line
                // *****Copie dfrom spartan command line
                //else if (StringUtils.ParseCmdArg(arg, BuildFlags.Debug, ref temp))
                //{
                //    BuildConfiguration = BuildConfiguration.Debug;
                //}
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.CoordProgram, ref temp))
                {
                    if (ProgramFunction != ProgramFunction.None)
                        throw new Exception("Multiple program functions (/a or /c) specified.");
                    ProgramFunction = ProgramFunction.Coordinator;
                    SpartanGlobals.LogFileName = "coordinator.log";
                }
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.AgentProgram, ref temp))
                {
                    if (ProgramFunction != ProgramFunction.None)
                        throw new Exception("Multiple program functions (/a or /c) specified.");
                    ProgramFunction = ProgramFunction.Agent;
                    SpartanGlobals.LogFileName = "agent.log";
                }
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.Clean, ref temp))
                {
                    CompileOptions.Add(CompileOption.Clean);
                }
                //else if (StringUtils.ParseCmdArg(arg, BuildFlags.ProjectFile, ref temp))
                //{
                //    ProjectsFileName = temp;
                //}
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.BuildId, ref temp))
                {
                    UserSuppliedBuildId = Convert.ToInt32(temp);
                }
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.AgentName, ref temp))
                {
                    CoordinatorAgentInfos.Add(new AgentInfo() { Name = temp });
                }
                else if (StringUtils.ParseCmdArg(arg, BuildFlags.HideConsoleWindow, ref temp))
                {
                    Win32Utils.ShowConsole(false);
                }

                else
                    Errors.Add("Warn: [BuildConfig] Possible unrecognized switch " + arg + " not recognized.");

            }
        }

    }
}
