﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Proteus;

namespace Spartan
{
    public class SpartanGlobals
    {
        #region Public: Build Variables

        public static bool GlobalsInitialized = false;
        //public static string SpartanRootDirectory;  //Very important 20151111 - get this from _objbuild config from projects file

        public static string LogFileName; // get this from build c


        public static string AgentServiceName = "RomulusService";//The name recognized by windows management. This is both the service name AND the .exe name
        public static string CoordinatorExeName = "CoordCmd";//The name recognized by windows management. // Sort of fucked up
        public static string CoordinatorVsHostExeName = "AgentCmd.vshost";//The name recognized by windows managementt. // Sort of fucked up
        public static string AgentExeName = "AgentCmd";
        public static string RomulusExeName = "Romulus";
        public static string BuildGuiExeName = "BuildGui";
        public static string BuildGuiVsHostExeName = "BuildGui.vshost";

        public static string AgentServiceDisplayName = "Romulus Build Agent";
        public static string AgentServiceDisplayDescription = "Romulus Build Agent Service";

        public const int CompilerThreadTimeoutInMilliseconds = (1000) * (60) * (10);
        public const int GlobalAgentIdleWaitInSeconds = 20;
        public const int GlobalAgentIdleTimeInMilliseconds = 200; //**this needs to be faster because the BUILD GUI polls the agents every second.
        public const int SendAndRecvTimeout = 60000;
        public const int CoordinatorToAgentConnectionInterval = 15000;//This cuases a huge slowdown
        public const int CoordinatorConnectionAttemptTimeout = 100;
        public const int CoordinatorDisconnectAttemptTimeout = 100;
        public const int MaxNumServerConnections = 100000000;
        public const int AgentBuildInterval = 10;   // Interval for agent to sleep between thread loops.


        #endregion



        #region Public: Network Variables

        public const string MsgHeaderBuildRequest       = "BRQ00000";
        public const string MsgHeaderFail               = "FAIL0000"; 
        public const string MsgHeaderOkToCompileFile    = "CFL00000"; 
        public const string MsgHeaderQueryAvailable     = "AVL00000";
        public const string MsgHeaderCompileComplete    = "CCP00000";
        public const string MsgStopCompilation          = "STC00000";
        public const string MsgQueryRequestStatus       = "QRS00000";
        public const string MsgAgentStatusInfo          = "APS00000";
        public const string MsgUnreserveProcessor       = "UNRS0000";
        public const string MsgResetGlobalCompileState  = "GRSP0000";
        public const string MsgRestartServer            = "RS000000";

        public const int ClientRecvPort = 58484;
        public const int ClientSendPort = 58485;
        public const int ServerRecvPort = 58486;
        public const int ServerSendPort = 58487;//**see also buildmonitor globals.

        public static Dictionary<PacketType, string> PacketHeaderDictionary = new Dictionary<PacketType, string>();

        #endregion





        #region Public: Network Methods

        public static string PacketTypeToMsgHeader(PacketType pt)
        {
            string value;
            if (PacketHeaderDictionary.TryGetValue(pt, out value) == false)
                throw new NotImplementedException();
            return value;
        }
        public static PacketType GetPacketType(ref string strBuffer)
        {
            string strCode = NetworkUtils.ParseDataChunk(ref strBuffer, 8);

            PacketType myValue = PacketHeaderDictionary.FirstOrDefault(x => x.Value == strCode).Key;

            if (myValue == PacketType.InvalidPacketType)
                throw new NotImplementedException();
            return myValue;
        }
        public static BroSourceFile ReadFileFromStream(ref string buffer, NetworkMode eNetworkMode)
        {
            BroSourceFile f = new BroSourceFile();

            f.FileBranchName = NetworkUtils.ReadStringFromStream(ref buffer, eNetworkMode).Trim();
            int len = NetworkUtils.UnpackInt(ref buffer, eNetworkMode);

            f.FileData = NetworkUtils.ReadStringFromStream(ref buffer, eNetworkMode);

            if (f.FileData.Length != len)
                throw new Exception(" File update length was invalid. got " + f.FileData.Length + " expected " + len);

            return f;
        }
        public static AgentCommand ReadCommandFromStream(ref string buffer, NetworkMode eNetworkMode)
        {
            AgentCommand ac = new AgentCommand();
            ac.ClientReservationId = NetworkUtils.UnpackInt(ref buffer, eNetworkMode);
            ac.CommandText = NetworkUtils.UnpackString(ref buffer, eNetworkMode);
            ac.OutFileName = NetworkUtils.UnpackString(ref buffer, eNetworkMode);
            return ac;
        }
       
        #endregion

        #region Public: Command Methods

        public static void InitializeGlobals(RomulusCommandLineArgs args)
        {
            PacketHeaderDictionary.Add(PacketType.BuildRequest, MsgHeaderBuildRequest);
            PacketHeaderDictionary.Add(PacketType.Error, MsgHeaderFail);
            PacketHeaderDictionary.Add(PacketType.OkToCompileFile, MsgHeaderOkToCompileFile);
            PacketHeaderDictionary.Add(PacketType.AgentStateQueryAvailable, MsgHeaderQueryAvailable);
            PacketHeaderDictionary.Add(PacketType.CompileRequestComplete, MsgHeaderCompileComplete);
            PacketHeaderDictionary.Add(PacketType.StopCompilation, MsgStopCompilation);
            PacketHeaderDictionary.Add(PacketType.QueryRequestStatus, MsgQueryRequestStatus);
            PacketHeaderDictionary.Add(PacketType.AgentStatusInfo, MsgAgentStatusInfo);
            PacketHeaderDictionary.Add(PacketType.UnreserveProcessor, MsgUnreserveProcessor);
            PacketHeaderDictionary.Add(PacketType.ResetGlobalCompileState, MsgResetGlobalCompileState);
            PacketHeaderDictionary.Add(PacketType.RestartServer, MsgRestartServer);

            if (args.ProgramFunction == ProgramFunction.Coordinator)
                LogFileName = "coordinator.log";
            else if (args.ProgramFunction == ProgramFunction.Agent)
                LogFileName = "agent.log";
            else
                throw new Exception("Invalid program function specified.");
            //string errTxt = SpartanGlobals.ParseArgs(args);//must parse first to determine if agent.

            Proteus.Proteus.Initialize(LogFileName);

            if (args.Errors.Count > 0)
                foreach(string err in args.Errors)
                    Globals.Logger.LogWarn(err);

            //if (ProgramFunction == ProgramFunction.Coordinator)
            //{
            //    ProjectsFile
            //    //Note: Only coordinator uses build config now.  We used to use it for agent.
            //    BuildConfig.ParseArgs(args);
            //    BuildConfig.LoadConfig();
            //}
            //if(string.IsNullOrEmpty(errTxt)==false)
            //    Globals.Logger.LogWarn(errTxt);

            GlobalsInitialized = true;
        }
        //public static string ParseArgs(List<string> args)
        //{
        //    string temp = string.Empty;
        //    string tstr = string.Empty;
        //    string errorTxt = string.Empty; // can't log in this method as logger isn't created yet

        //    foreach (string str in args)
        //    {
        //        tstr = str.Trim();
                

        //        else
        //        {
        //            errorTxt += "Possible Unrecognized argument " + str + " ";
        //        }
        //    }
        //    return errorTxt;
        //}
        public static void ValidateForBuild(RomulusCommandLineArgs args, ProjectsFile pf)
        {
            string strErrors = string.Empty;

            if (args.ProgramFunction == ProgramFunction.None)
                strErrors += " * Program function was not set.  Must specify /c for coordinator or /a for agent.";

            if (args.ProgramFunction == ProgramFunction.Coordinator)
            {
                if (String.IsNullOrEmpty(pf.GetBuildConfig().GlobalBuildConfiguration.BuildPlatform))
                    strErrors += " * Build Configuration platform is not set. (use /cp switch) \n";
                if (String.IsNullOrEmpty(pf.GetBuildConfig().GlobalBuildConfiguration.ConfigurationName))
                    strErrors += " * Build Configuration name is not set. (use /cn switch) \n";
            }

            if (strErrors != string.Empty)
                Globals.Logger.LogError(
                    " Could not validate build configuration. \n" +
                    " The following errors must be corrected before build can proceed.:\n"
                    + strErrors,
                    true
                    );
        }


        private static bool DevOrRealProcessIsRunning(string realName, string devName)
        {
            System.Diagnostics.Process[] procs;

            procs = System.Diagnostics.Process.GetProcessesByName(realName);
            if (procs.Length > 0)
                return true;

            procs = System.Diagnostics.Process.GetProcessesByName(devName);
            if (procs.Length > 0)
                return true;

            return false;
        }
        public static bool BuildGuiIsRunning()
        {
            return DevOrRealProcessIsRunning(SpartanGlobals.BuildGuiExeName, SpartanGlobals.BuildGuiVsHostExeName);
        }
        public static bool CoordinatorIsRunning()
        {
            return DevOrRealProcessIsRunning(SpartanGlobals.CoordinatorExeName, SpartanGlobals.CoordinatorVsHostExeName);
        }
        #endregion



    }
}
